@@ -383,7 +383,7 @@ gdm_activate_session_by_id (GDBusConnection *connection,
 
 gboolean
 gdm_get_login_window_session_id (const char  *seat_id,
-		                 char       **session_id)
+                                 char       **session_id)
 {
         gboolean   ret;
         int        res, i;
@@ -408,10 +408,8 @@ gdm_get_login_window_session_id (const char  *seat_id,
 
                 res = sd_session_get_class (sessions[i], &service_class);
                 if (res < 0) {
-                        if (res == -ENOENT) {
-                                free (service_class);
+                        if (res == -ENXIO)
                                 continue;
-                        }
 
                         g_debug ("failed to determine class of session %s: %s", sessions[i], strerror (-res));
                         ret = FALSE;
@@ -427,6 +425,9 @@ gdm_get_login_window_session_id (const char  *seat_id,
 
                 ret = sd_session_get_state (sessions[i], &state);
                 if (ret < 0) {
+                        if (res == -ENXIO)
+                                continue;
+
                         g_debug ("failed to determine state of session %s: %s", sessions[i], strerror (-res));
                         ret = FALSE;
                         goto out;
@@ -440,6 +441,9 @@ gdm_get_login_window_session_id (const char  *seat_id,
 
                 res = sd_session_get_service (sessions[i], &service_id);
                 if (res < 0) {
+                        if (res == -ENXIO)
+                                continue;
+
                         g_debug ("failed to determine service of session %s: %s", sessions[i], strerror (-res));
                         ret = FALSE;
                         goto out;
